<?php

namespace Drupal\fileslog;

use Drupal\Component\Serialization\Json;
use Drupal\Core\File\FileSystemInterface;
use Drupal\file\Entity\File;
use Drupal\fileslog\Form\FileslogFilterForm;
use Drupal\fileslog\Logger\FilesLog;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * The manager for files logs.
 */
class FilesLogManager {

  /**
   * The file system.
   *
   * @var \Drupal\Core\File\FileSystemInterface
   */
  protected FileSystemInterface $fileSystem;

  /**
   * The current request.
   *
   * @var \Symfony\Component\HttpFoundation\Request
   */
  protected Request $request;

  /**
   * The total amount of logs.
   *
   * @var int
   */
  protected int $total = 0;

  /**
   * Constructs the manager.
   *
   * @param \Drupal\Core\File\FileSystemInterface $file_system
   *   The file system.
   */
  public function __construct(FileSystemInterface $file_system, RequestStack $request_stack) {
    $this->fileSystem = $file_system;
    $this->request = $request_stack->getCurrentRequest();
  }

  /**
   * Returns the log files.
   *
   * @param array $filters
   *   The filters to filter the results by.
   * @param int $amount
   *   The amount of items to show.
   * @param int $offset
   *   The offset to slice results with.
   *
   * @return array
   */
  public function getLogFiles(array $filters = [], int $amount = FilesLogInterface::ITEMS_PER_PAGE, int $offset = 0): array {
    // Retrieve all .json files from the directory.
    $files = $this->fileSystem->scanDirectory(FilesLogInterface::DIRECTORY, '/(.?)\.json/', ['recurse' => FALSE]);
    // Sort the log files in reverse, so the latest show up first.
    krsort($files);
    $levels = $filters['severity'] ?? [];
    if (count($levels) > 0) {
      $files = array_filter($files, function ($file) use ($levels) {
        foreach ($levels as $filter_level) {
          [$timestamp, $log_level] = explode('-', $file->name);
          if ($log_level == $filter_level) {
            return TRUE;
          }
        }
        return FALSE;
      });
    }

    $this->total = count($files);

    if ($amount === 0) {
      return $files;
    }
    return array_slice($files, $offset, $amount);
  }

  /**
   * Returns the logs.
   *
   * @param array $filters
   *   The filters to filter the results by.
   * @param int $amount
   *   The amount of items to show.
   * @param int $offset
   *   The offset to slice results with.
   *
   * @return \Drupal\fileslog\FilesLogEntry[]
   */
  public function getLogs(array $filters = [], int $amount = FilesLogInterface::ITEMS_PER_PAGE, int $offset = 0): array {
    $files = $this->getLogFiles($filters, $amount, $offset);
    $logs = [];

    foreach ($files as $uri => $file) {
      $file_contents = file_get_contents($this->fileSystem->realpath($uri));
      $log = Json::decode($file_contents);
      $logs[$uri] = FilesLogEntry::createFromJson($log, $file->name);
    }

    return $logs;
  }

  /**
   * Returns a single log.
   *
   * @param string $filename
   *   The filename.
   *
   * @return \Drupal\fileslog\FilesLogEntry
   */
  public function getLog(string $filename): FilesLogEntry {
    $uri = FilesLogInterface::DIRECTORY . '/' . $filename . '.json';
    $file_contents = file_get_contents($this->fileSystem->realpath($uri));
    $log = Json::decode($file_contents);

    return FilesLogEntry::createFromJson($log, $filename);
  }

  /**
   * Returns the total logs count.
   *
   * @return int
   */
  public function getTotalLogsCount(): int {
    return $this->total;
  }

  /**
   * Deletes all the logs.
   *
   * @return void
   */
  public function deleteLogs() {
    foreach ($this->getLogFiles() as $uri => $file) {
      $this->fileSystem->unlink($uri);
    }
  }

}
